﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Students
{
    static class Extensions
    {
        /// <summary>
        /// Randomly grades students.
        /// This method is purely fictional and any similarity with reality is purely accidental :)
        /// </summary>
        /// <param name="students">Students to be graded</param>
        public static void RandomlyGradeStudents(this IEnumerable<Student> students)
        {
            Random rnd = new Random();
            foreach (var student in students)
            {
                // 10 pity points for each :)
                student.IncreaseScore(10 + rnd.Next(30));
            }
        }

        /// <summary>
        /// Performs an action on each element.
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="xs">Collection to be iterated thru</param>
        /// <param name="f">Action to be called for each element</param>
        public static void Run<T>(this IEnumerable<T> xs, Action<T> f)
        {
            foreach (var x in xs)
            {
                f(x);
            }
        }

        /// <summary>
        /// Same as Run, only composable.
        /// </summary>
        /// <example>xs.OrderBy(x => x.A).Do(x => Console.WriteLine(x)).Select(x => x * x)</example>
        /// <typeparam name="T"></typeparam>
        /// <param name="xs"></param>
        /// <param name="f">Action to be called for each element</param>
        /// <returns>Unmodified enumerable</returns>
        public static IEnumerable<T> Do<T>(this IEnumerable<T> xs, Action<T> f)
        {
            foreach (var x in xs)
            {
                // perform
                f(x);
                
                // 'yield return' returns the current element halts the execution of a method until the next element of the enumerable is requested
                // in effect, this reconstructs the original enumerable
                yield return x;
            }
        }
    }
}
