﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Xml.Linq;

namespace Students
{
    /// <summary>
    /// A class representing students
    /// </summary>
    class Student
    {
        public enum Genders
        {
            Male,
            Female
        }

        public enum Grades
        {
            A,
            B,
            C,
            D,
            E,
            F
        }

        public string Name { get; private set; }
        public string Surname { get; private set; }

        //public string FullName
        //{
        //    get
        //    {
        //        return Name + " " + Surname;
        //    }
        //}

        public int UCO { get; private set; }
        public string Study { get; private set; }
        public int GroupNumber { get; private set; }
        public Genders Gender { get; private set; }

        public int Score { get; private set; }

        /// <summary>
        /// Increses score of a student by the given amount.
        /// Maximum score is 40.
        /// </summary>
        /// <param name="by">Non-negative number</param>
        /// <returns>Student's new score</returns>
        public int IncreaseScore(int by)
        {
            if (by < 0) throw new ArgumentException("Score can't be increased by a negative value!");
            if (Score + by > 40) throw new InvalidOperationException("Can't increase score above 40!");

            Score += by;

            return Score;
        }

        /// <summary>
        /// Grades a student based on his current score
        /// </summary>
        public Grades Grade 
        {
            get
            {
                if (Score < 23) return Grades.F;
                if (Score < 27) return Grades.E;
                if (Score < 31) return Grades.D;
                if (Score < 35) return Grades.C;
                if (Score < 38) return Grades.B;
                return Grades.A;
            }
        }

        /// <summary>
        /// Creates a new instance of Student from XML representation
        /// </summary>
        /// <remarks>For XML format specification use your imagination</remarks>
        /// <param name="element">XML representation of a student</param>
        /// <returns>New Student</returns>
        public static Student FromXML(XElement element)
        {
            return new Student
            {
                Name = element.Attribute("Name").Value,
                Surname = element.Attribute("Surname").Value,
                UCO = int.Parse(element.Attribute("UCO").Value),
                Study = element.Attribute("Study").Value,
                GroupNumber = int.Parse(element.Attribute("GroupNumber").Value),
                Gender = (Genders)Enum.Parse(typeof(Genders), element.Attribute("Gender").Value, true)
            };
        }

        private Student()
        {
        }

        /// <summary>
        /// Constructor ...
        /// </summary>
        /// <param name="name"></param>
        /// <param name="surname"></param>
        /// <param name="uco"></param>
        /// <param name="study"></param>
        /// <param name="groupNumber"></param>
        /// <param name="gender"></param>
        public Student(string name, string surname, int uco, string study, int groupNumber, Genders gender)
        {
            this.Name = name;
            this.Surname = surname;
            this.UCO = uco;
            this.Study = study;
            this.GroupNumber = groupNumber;
            this.Gender = gender;
        }

        public override string ToString()
        {
            return string.Format("{0} {1}, {2}, {3}, group {4}", Name, Surname, UCO, Study, GroupNumber);
        }
    }
}
